//Roller blinds with local push button and home possition

#include <Relay.h>
#include <SPI.h>
#include <EEPROM.h>  
#include <RF24.h>
#include <AccelStepper.h>

#define HOMEBUTTON_PIN  3  // Arduino Digital I/O pin number for button
#define BUTTON_PIN  6  // Arduino Digital I/O pin number for control button (with 10k pulldown) 
#define STEP_PIN  5  // Arduino Digital I/O pin number for Step pin on Easy driver board
#define DIR_PIN  4  // Arduino Digital I/O pin number for Dir pin on Easy driver board
#define SLEEP_PIN 7 // Arduino Digital I/O pin number for Sleep pin on Easy driver board
#define RADIO_ID 7
#define CHILD_ID 1   // Id of the sensor child

#define PIN_ON 0
#define PIN_OFF 1

int buttVal;
int previousVal;
int homebuttonVal;
volatile int dirVal=0;
volatile int olddirVal=0;
int state;
int switched = 0;
long pressTime = 0;    // the last time the button pin was pressed
long debounce = 200;   // the debounce time, increase if the output flickers

AccelStepper stepper1(1, STEP_PIN, DIR_PIN);

byte hBval;

Sensor gw;

void setup(){
  //Serial.begin(BAUD_RATE);
  //Serial.print("Serial begin");
  
  gw.begin(RADIO_ID);
  //Serial.print("GW begin");
  // Send the Sketch Version Information to the Gateway
  gw.sendSketchInfo("Blinds", "2.0");
  //Serial.print("Send Info");
  // Register all sensors to gw (they will be created as child devices)
  gw.sendSensorPresentation(CHILD_ID, S_COVER);
  //Serial.print("Send pressentation");
  dirVal = olddirVal = atol(gw.getStatus(CHILD_ID, V_VAR1));
  //Serial.println(dirVal);
  stepper1.setMaxSpeed(2500); //nice and slow for testing
  //stepper1.moveTo(-3200);
  stepper1.setAcceleration(800);
  pinMode(HOMEBUTTON_PIN, INPUT);
  pinMode(BUTTON_PIN, INPUT);
  pinMode(SLEEP_PIN, OUTPUT);
  digitalWrite(HOMEBUTTON_PIN, HIGH);
  digitalWrite(BUTTON_PIN, HIGH);
  digitalWrite(SLEEP_PIN, LOW);
  buttVal = digitalRead(BUTTON_PIN); //read initial state
  //stepperHome(); //runs routine to home motor
}
void loop(){
  
   if (gw.messageAvailable()) {
    //Serial.print("New message from GW"); 
    // ot new messsage from gw
    message_s message = gw.getMessage(); 
    setRelayStatus(message);
  }
  
  buttVal = digitalRead(BUTTON_PIN);      // read input value and store it in val
  homebuttonVal = digitalRead(HOMEBUTTON_PIN);      // read input value and store it in val

  if (buttVal == LOW && previousVal == HIGH) {
      // Start counter from when button was pressed
      pressTime = millis();
      switched = 0;
   }

   if (buttVal == LOW && switched == 0 && millis() - pressTime > debounce) {
     // Switch state if button pressed more than 200 msec
    if (dirVal == 1) {
      digitalWrite(SLEEP_PIN, PIN_OFF); 
      delay(5);
      stepper1.moveTo(-18000);
      stepper1.runToPosition();
      dirVal = 0;
      gw.sendVariable(CHILD_ID, V_VAR1, dirVal);
      gw.sendVariable(CHILD_ID, V_DIMMER, 0);
      digitalWrite(SLEEP_PIN, PIN_ON);
    } else {
      digitalWrite(SLEEP_PIN, PIN_OFF); 
      delay(5);
      stepper1.moveTo(0);
      stepper1.runToPosition();
      dirVal = 1;
      gw.sendVariable(CHILD_ID, V_VAR1, dirVal);
      gw.sendVariable(CHILD_ID, V_DIMMER, 100);
      digitalWrite(SLEEP_PIN, PIN_ON);
    }
     
     switched = 1; // No more switches until button is released
    }
    
     if (homebuttonVal == LOW) {
        stepperHome(); //runs routine to home motor
   }
   
    previousVal = buttVal;
    
}
//contributed by Runaway Pancake 9/2/13
void stepperHome(){ //this routine should run the motor
  hBval = digitalRead(HOMEBUTTON_PIN);
  digitalWrite(SLEEP_PIN, PIN_OFF); 
  delay(5);
  while (hBval == LOW)
  {
    //Serial.print("Stuck in home function");
    digitalWrite(SLEEP_PIN, PIN_OFF); 
      delay(5);
    //backwards slowly till it hits the switch and stops
    stepper1.moveTo(2000);
    stepper1.run();
    hBval = digitalRead(HOMEBUTTON_PIN);
  }
  stepper1.setCurrentPosition(0); //should set motor position to zero and go back to main routine
      dirVal = 1;
      gw.sendVariable(CHILD_ID, V_VAR1, dirVal);
      gw.sendVariable(CHILD_ID, V_DIMMER, 100);
  digitalWrite(SLEEP_PIN, PIN_ON);
}

void setRelayStatus(message_s message) {
  if (message.header.type==V_DIMMER) { // 
     int IncomingControl = atoi(message.data);
     //Serial.print("Message: ");
     //Serial.println(IncomingControl);
     if (IncomingControl == 0) {
      digitalWrite(SLEEP_PIN, PIN_OFF); 
      delay(5);
      stepper1.moveTo(-18000);
      stepper1.runToPosition();
      dirVal = 0;
      gw.sendVariable(CHILD_ID, V_VAR1, dirVal);
      gw.sendVariable(CHILD_ID, V_DIMMER, IncomingControl);
      digitalWrite(SLEEP_PIN, PIN_ON);
     } 
     if (IncomingControl == 100) {
      digitalWrite(SLEEP_PIN, PIN_OFF); 
      delay(5);
      stepper1.moveTo(0);
      stepper1.runToPosition();
      dirVal = 1;
      gw.sendVariable(CHILD_ID, V_VAR1, dirVal);
      gw.sendVariable(CHILD_ID, V_DIMMER, IncomingControl);
      digitalWrite(SLEEP_PIN, PIN_ON); 
      
     }
    
    }
}
